@@ -58,7 +58,8 @@ setMethod (
 ## operation between an db.obj and a single value
 .compare <- function (e1, e2, cmp, data.types,
                       prefix = "", res.type = "boolean",
-                      cast = "::double precision")
+                      cast = "::double precision", res.udt = "bool",
+                      restore.array = TRUE)
 {
     if (is(e1, "db.data.frame")) e1 <- e1[,]
 
@@ -78,37 +79,77 @@ setMethod (
     else where.str <- ""
 
     sort <- .generate.sort(e1)
-    
+
     expr <- rep("", length(names(e1)))
     col.data_type <- rep("", length(names(e1)))
     col.udt_name <- rep("", length(names(e1)))
-    col.name <- rep("", length(names(e1)))
-    for (i in seq_len(length(names(e1)))) {
+    ## col.name <- rep("", length(names(e1)))
+    col.name <- paste(names(e1), "_opr", sep = "")
+    i <- 1
+    len <- length(names(e1))
+    while (i <= len) {
         col.data_type[i] <- res.type
-        col.udt_name[i] <- res.type
-        col.name[i] <- paste(names(e1)[i], "_opr", sep = "")
-        if (e1@.col.data_type[i] %in% data.types || is.na(data.types)) {
+        col.udt_name[i] <- res.udt
+        ## col.name[i] <- paste(names(e1)[i], "_opr", sep = "")
+        s <- e2[count %% l + 1]
+        whole.array <- is.character(s) && grepl("^\\{", .strip(s)) && grepl("\\}$", .strip(s))
+        if (e1@.col.data_type[i] %in% data.types || is.na(data.types) || whole.array) {
+            e2.str <- (if (e2[count %% l + 1] == "") "" else paste(
+                "(", e2[count %% l + 1], ")", sep = ""))
             expr[i] <- paste(prefix, "(", e1@.expr[i], ")", cast,
-                             cmp, e2[count %% l + 1], sep = "")
+                             cmp, e2.str, sep = "")
             count <- count + 1
+            i <- i + 1
         } else if (e1@.col.data_type[i] == "array") {
             tmp <- .get.array.elements(e1@.expr[i], tbl, where.str,
                                        conn.id(e1))
             tn <- length(tmp)
-            expr[i] <- paste("array[", paste(prefix, "(", tmp, ")", cast,
-                                             cmp,
-                                             e2[(count + seq(tn) - 1) %% l + 1],
-                                             sep = "",
-                                             collapse = ", "), "]",
-                             sep = "")
+            ## e2.str <- (if (e2[(count + seq(tn) - 1) %% l + 1] == "") ""
+            ## else paste("(", e2[count %% l + 1], ")", sep = ""))
+
+            if (cast == "")
+                e2.str <- e2[(count + seq(tn) - 1) %% l + 1]
+            else
+                e2.str <- paste("(", e2[(count + seq(tn) - 1) %% l + 1], ")",
+                                sep = "")
+
+            if (restore.array) {
+                expr[i] <- paste("array[", paste(prefix, "(", tmp, ")", cast,
+                                                 cmp, e2.str, sep = "",
+                                                 collapse = ", "), "]",
+                                 sep = "")
+                col.data_type[i] <- "array"
+                col.udt_name[i] <- paste("_", res.udt, sep = "")
+                i <- i + 1
+            } else {
+                expr.t <- paste(prefix, "(", tmp, ")", cast, cmp, e2.str,
+                                sep = "")
+                expr <- c(expr[seq_len(i-1)], expr.t,
+                          expr[(i+1)+seq_len(length(expr)-i)])
+                col.data_type <- c(col.data_type[seq_len(i-1)],
+                                   rep("boolean", tn),
+                                   col.data_type[(i+1)+
+                                                 seq_len(length(
+                                                     col.data_type)-i)])
+                col.udt_name<- c(col.udt_name[seq_len(i-1)],
+                                   rep("bool", tn),
+                                   col.udt_name[(i+1)+
+                                                seq_len(length(
+                                                    col.udt_name)-i)])
+                col.name <- c(col.name[seq_len(i-1)],
+                              paste(col.name[i], "_", seq_len(tn), sep=""),
+                              col.name[(i+1)+seq_len(length(col.name)-i)])
+                i <- i + tn
+                len <- len + tn - 1
+            }
             count <- count + tn
-            col.data_type[i] <- "array"
-            col.udt_name[i] <- paste("_", res.type)
         } else {
             expr[i] <- "NULL"
+            count <- count + 1
+            i <- i + 1
         }
     }
-   
+
     expr.str <- paste(expr, paste("\"", col.name, "\"", sep = ""),
                       sep = " as ", collapse = ", ")
     new("db.Rquery",
@@ -125,7 +166,8 @@ setMethod (
         .where = e1@.where,
         .is.factor = rep(FALSE, length(col.name)),
         .factor.suffix = rep("", length(col.name)),
-        .sort = sort)
+        .sort = sort,
+        .dist.by = e1@.dist.by)
 }
 
 ## -----------------------------------------------------------------------
@@ -150,7 +192,9 @@ setMethod (
     ">",
     signature(e1 = "db.obj", e2 = "numeric"),
     function (e1, e2) {
-        .compare(e1, e2, " > ", .num.types)
+        res <- .compare(e1, e2, " > ", .num.types)
+        if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+        res
     },
     valueClass = "db.Rquery")
 
@@ -170,7 +214,9 @@ setMethod (
     "<",
     signature(e1 = "db.obj", e2 = "numeric"),
     function (e1, e2) {
-        .compare(e1, e2, " < ", .num.types)
+        res <- .compare(e1, e2, " < ", .num.types)
+        if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+        res
     },
     valueClass = "db.Rquery")
 
@@ -190,7 +236,9 @@ setMethod(
     ">=",
     signature(e1 = "db.obj", e2 = "numeric"),
     function (e1, e2) {
-        .compare(e1, e2, " >= ", .num.types)
+        res <- .compare(e1, e2, " >= ", .num.types)
+        if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+        res
     },
     valueClass = "db.Rquery")
 
@@ -210,7 +258,9 @@ setMethod (
     "<=",
     signature(e1 = "db.obj", e2 = "numeric"),
     function (e1, e2) {
-        .compare(e1, e2, " <= ", .num.types)
+        res <- .compare(e1, e2, " <= ", .num.types)
+        if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+        res
     },
     valueClass = "db.Rquery")
 
@@ -230,7 +280,9 @@ setMethod (
     "==",
     signature(e1 = "db.obj", e2 = "numeric"),
     function (e1, e2) {
-        .compare(e1, e2, " = ", .num.types)
+        res <- .compare(e1, e2, " = ", .num.types)
+        if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+        res
     },
     valueClass = "db.Rquery")
 
@@ -246,11 +298,57 @@ setMethod (
 
 ## --
 
+setMethod (
+    "==",
+    signature(e1 = "db.obj", e2 = "logical"),
+    function (e1, e2) {
+        res <- .compare(e1, e2, " = ", "boolean", cast = "")
+        if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+        res
+    },
+    valueClass = "db.Rquery")
+
+## --
+
+setMethod (
+    "==",
+    signature(e1 = "logical", e2 = "db.obj"),
+    function (e1, e2) {
+        e2 == e1
+    },
+    valueClass = "db.Rquery")
+
+## --
+
+setMethod (
+    "!=",
+    signature(e1 = "db.obj", e2 = "logical"),
+    function (e1, e2) {
+        res <- .compare(e1, e2, " <> ", "boolean", cast = "")
+        if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+        res
+    },
+    valueClass = "db.Rquery")
+
+## --
+
+setMethod (
+    "!=",
+    signature(e1 = "logical", e2 = "db.obj"),
+    function (e1, e2) {
+        e2 != e1
+    },
+    valueClass = "db.Rquery")
+
+## --
+
 setMethod (
     "!=",
     signature(e1 = "db.obj", e2 = "numeric"),
     function (e1, e2) {
-        .compare(e1, e2, " <> ", .num.types)
+        res <- .compare(e1, e2, " <> ", .num.types)
+        if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+        res
     },
     valueClass = "db.Rquery")
 
@@ -266,12 +364,38 @@ setMethod (
 
 ## -----------------------------------------------------------------------
 
+## convert string to time types
+.replace.timestamp <- function (e1, res, s, op)
+{
+    types <- col.types(e1)
+    time.types <- c("timestamp", "time", "date", "interval")
+    for (i in seq_len(length(types))) {
+        for (t in time.types) {
+            if (grepl(t, types[i])) {
+                res@.expr[i] <- paste(e1@.expr[i], op, s, "::", t, sep = "")
+                res@.content <- gsub(paste("NULL as \"", res@.col.name[i],
+                                           "\"", sep = ""),
+                                paste(res@.expr[i], " as \"",
+                                      res@.col.name[i], "\"", sep = ""),
+                                res@.content)
+                break
+            }
+        }
+    }
+    res
+}
+
+## ----------------------------------------------------------------------
+
 setMethod (
     ">",
     signature(e1 = "db.obj", e2 = "character"),
     function (e1, e2) {
         e2 <- paste("'", .strip(e2, "'"), "'", sep = "")
-        .compare(e1, e2, " > ", .txt.types, cast = "")
+        res <- .compare(e1, e2, " > ", .txt.types, cast = "")
+        res <- .replace.timestamp(e1, res, e2, " > ")
+        if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+        res
     },
     valueClass = "db.Rquery")
 
@@ -292,7 +416,10 @@ setMethod (
     signature(e1 = "db.obj", e2 = "character"),
     function (e1, e2) {
         e2 <- paste("'", .strip(e2, "'"), "'", sep = "")
-        .compare(e1, e2, " < ", .txt.types, cast = "")
+        res <- .compare(e1, e2, " < ", .txt.types, cast = "")
+        res <- .replace.timestamp(e1, res, e2, " < ")
+        if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+        res
     },
     valueClass = "db.Rquery")
 
@@ -313,7 +440,10 @@ setMethod (
     signature(e1 = "db.obj", e2 = "character"),
     function (e1, e2) {
         e2 <- paste("'", .strip(e2, "'"), "'", sep = "")
-        .compare(e1, e2, " >= ", .txt.types, cast = "")
+        res <- .compare(e1, e2, " >= ", .txt.types, cast = "")
+        res <- .replace.timestamp(e1, res, e2, " >= ")
+        if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+        res
     },
     valueClass = "db.Rquery")
 
@@ -334,7 +464,10 @@ setMethod (
     signature(e1 = "db.obj", e2 = "character"),
     function (e1, e2) {
         e2 <- paste("'", .strip(e2, "'"), "'", sep = "")
-        .compare(e1, e2, " <= ", .txt.types, cast = "")
+        res <- .compare(e1, e2, " <= ", .txt.types, cast = "")
+        res <- .replace.timestamp(e1, res, e2, " <= ")
+        if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+        res
     },
     valueClass = "db.Rquery")
 
@@ -355,7 +488,10 @@ setMethod (
     signature(e1 = "db.obj", e2 = "character"),
     function (e1, e2) {
         e2 <- paste("'", .strip(e2, "'"), "'", sep = "")
-        .compare(e1, e2, " = ", .txt.types, cast = "")
+        res <- .compare(e1, e2, " = ", .txt.types, cast = "")
+        res <- .replace.timestamp(e1, res, e2, " = ")
+        if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+        res
     },
     valueClass = "db.Rquery")
 
@@ -376,7 +512,10 @@ setMethod (
     signature(e1 = "db.obj", e2 = "character"),
     function (e1, e2) {
         e2 <- paste("'", .strip(e2, "'"), "'", sep = "")
-        .compare(e1, e2, " <> ", .txt.types, cast = "")
+        res <- .compare(e1, e2, " <> ", .txt.types, cast = "")
+        res <- .replace.timestamp(e1, res, e2, " <> ")
+        if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+        res
     },
     valueClass = "db.Rquery")
 
@@ -396,7 +535,10 @@ setMethod (
     "&",
     signature(e1 = "db.obj", e2 = "logical"),
     function (e1, e2) {
-        .compare(e1, e2, " and ", c("boolean"), res.type = "boolean", cast = "")
+        res <- .compare(e1, e2, " and ", c("boolean"), res.type = "boolean",
+                        cast = "")
+        if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+        res
     },
     valueClass = "db.Rquery")
 
@@ -406,7 +548,10 @@ setMethod (
     "|",
     signature(e1 = "db.obj", e2 = "logical"),
     function (e1, e2) {
-        .compare(e1, e2, " or ", c("boolean"), res.type = "boolean", cast = "")
+        res <- .compare(e1, e2, " or ", c("boolean"), res.type = "boolean",
+                        cast = "")
+        if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+        res
     },
     valueClass = "db.Rquery")
 
@@ -435,7 +580,11 @@ setMethod (
     "+",
     signature(e1 = "db.obj", e2 = "numeric"),
     function (e1, e2) {
-        .compare(e1, e2, " + ", .num.types, res.type = "double precision")
+        res <- .compare(e1, e2, " + ", .num.types,
+                        res.type = "double precision",
+                        res.udt = "float8")
+        if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+        res
     },
     valueClass = "db.Rquery")
 
@@ -455,7 +604,11 @@ setMethod (
     "-",
     signature(e1 = "db.obj", e2 = "numeric"),
     function (e1, e2) {
-        .compare(e1, e2, " - ", .num.types, res.type = "double precision")
+        res <- .compare(e1, e2, " - ", .num.types,
+                        res.type = "double precision",
+                        res.udt = "float8")
+        if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+        res
     },
     valueClass = "db.Rquery")
 
@@ -465,8 +618,10 @@ setMethod (
     "-",
     signature(e1 = "numeric", e2 = "db.obj"),
     function (e1, e2) {
-        .compare(e2, e1, " + ", .num.types, prefix = "-",
-                 res.type = "double precision")
+        res <- .compare(e2, e1, " + ", .num.types, prefix = "-",
+                        res.type = "double precision", res.udt = "float8")
+        if (is(e2, "db.Rquery")) res@.is.agg <- e2@.is.agg
+        res
     },
     valueClass = "db.Rquery")
 
@@ -476,7 +631,28 @@ setMethod (
     "*",
     signature(e1 = "db.obj", e2 = "numeric"),
     function (e1, e2) {
-        .compare(e1, e2, " * ", .num.types, res.type = "double precision")
+        if (length(names(e1)) == 1 && e1@.col.data_type == "array" &&
+            length(e2) == 1) {
+            e1 <- e1[[names(e1)]]
+            res <- e1
+            madlib <- schema.madlib(conn.id(e1))
+            res@.expr <- paste(madlib, ".array_scalar_mult(", e1@.expr,
+                               "::double precision[], ",
+                               e2, "::double precision)", sep = "")
+            res@.col.name <- paste(e1@.col.name, "_opr", sep = "")
+            ## res@.content <- gsub("^select [^((?! as ).)]+\\S+ as",
+            res@.content <- gsub("^select .* as",
+                                 paste("select ", res@.expr, " as", sep = ""),
+                                 e1@.content)
+            if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+            return (res)
+        } else {
+            res <- .compare(e1, e2, " * ", .num.types,
+                            res.type = "double precision",
+                            res.udt = "float8")
+            if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+            res
+        }
     },
     valueClass = "db.Rquery")
 
@@ -496,7 +672,11 @@ setMethod (
     "/",
     signature(e1 = "db.obj", e2 = "numeric"),
     function (e1, e2) {
-        .compare(e1, e2, " / ", .num.types, res.type = "double precision", cast = "::double precision")
+        res <- .compare(e1, e2, " / ", .num.types,
+                        res.type = "double precision",
+                        cast = "::double precision", res.udt = "float8")
+        if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+        res
     },
     valueClass = "db.Rquery")
 
@@ -506,8 +686,10 @@ setMethod (
     "/",
     signature(e1 = "numeric", e2 = "db.obj"),
     function (e1, e2) {
-        .compare(e2, e1, " * ", .num.types, prefix = "1./",
-                 res.type = "double precision")
+        res <- .compare(e2, e1, " * ", .num.types, prefix = "1./",
+                        res.type = "double precision", res.udt = "float8")
+        if (is(e2, "db.Rquery")) res@.is.agg <- e2@.is.agg
+        res
     },
     valueClass = "db.Rquery")
 
@@ -517,7 +699,11 @@ setMethod (
     "^",
     signature(e1 = "db.obj", e2 = "numeric"),
     function (e1, e2) {
-        .compare(e1, e2, "^", .num.types, res.type = "double precision")
+        res <- .compare(e1, e2, "^", .num.types,
+                        res.type = "double precision",
+                        res.udt = "float8")
+        if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+        res
     },
     valueClass = "db.Rquery")
 
@@ -527,9 +713,11 @@ setMethod (
     "^",
     signature(e1 = "numeric", e2 = "db.obj"),
     function (e1, e2) {
-        .compare(e2, "", "", .num.types,
-                 prefix = paste(e1, "^", sep = ""),
-                 res.type = "double precision")
+        res <- .compare(e2, "", "", .num.types,
+                        prefix = paste(e1, "^", sep = ""),
+                        res.type = "double precision", res.udt = "float8")
+        if (is(e2, "db.Rquery")) res@.is.agg <- e2@.is.agg
+        res
     },
     valueClass = "db.Rquery")
 
@@ -539,7 +727,11 @@ setMethod (
     "%%",
     signature(e1 = "db.obj", e2 = "numeric"),
     function (e1, e2) {
-        .compare(e1, e2, " % ", .num.types, res.type = "double precision", cast = "::integer")
+        res <- .compare(e1, e2, " % ", .num.types,
+                        res.type = "double precision",
+                        cast = "::integer", res.udt = "float8")
+        if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+        res
     },
     valueClass = "db.Rquery")
 
@@ -549,9 +741,12 @@ setMethod (
     "%%",
     signature(e1 = "numeric", e2 = "db.obj"),
     function (e1, e2) {
-        .compare(e2, "", "", .num.types,
-                 prefix = paste(e1, "% "),
-                 res.type = "double precision", cast = "::integer")
+        res <- .compare(e2, "", "", .num.types,
+                        prefix = paste(e1, "% "),
+                        res.type = "double precision", cast = "::integer",
+                        res.udt = "float8")
+        if (is(e2, "db.Rquery")) res@.is.agg <- e2@.is.agg
+        res
     },
     valueClass = "db.Rquery")
 
@@ -561,7 +756,11 @@ setMethod (
     "%/%",
     signature(e1 = "db.obj", e2 = "numeric"),
     function (e1, e2) {
-        .compare(e1, as.integer(e2), " / ", .int.types, res.type = "integer", cast = "::integer")
+        res <- .compare(e1, as.integer(e2), " / ", .int.types,
+                        res.type = "integer", cast = "::integer",
+                        res.udt = "int4")
+        if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+        res
     },
     valueClass = "db.Rquery")
 
@@ -571,9 +770,12 @@ setMethod (
     "%/%",
     signature(e1 = "numeric", e2 = "db.obj"),
     function (e1, e2) {
-        .compare(e2, "", "", .num.types,
-                 prefix = paste(e1, "%"),
-                 res.type = "integer", cast = "::integer")
+        res <- .compare(e2, "", "", .num.types,
+                        prefix = paste(e1, "%"),
+                        res.type = "integer", cast = "::integer",
+                        res.udt = "int4")
+        if (is(e2, "db.Rquery")) res@.is.agg <- e2@.is.agg
+        res
     },
     valueClass = "db.Rquery")
 
@@ -582,9 +784,10 @@ setMethod (
 ## Arith operators for db.Rquery and db.Rquery
 
 ## operator for two db.obj objects
-.operate.two <- function (e1, e2, op, data.types, 
+.operate.two <- function (e1, e2, op, data.types,
                           res.type = "boolean",
-                          cast = "::double precision")
+                          cast = "::double precision",
+                          res.udt = "bool")
 {
     ## convert db.data.frame into db.Rquery
     if (is(e1, "db.data.frame")) e1 <- e1[,]
@@ -617,7 +820,7 @@ setMethod (
     if (e1@.parent != e2@.parent || !.eql.where(e1@.where, e2@.where))
         stop("How can you match the rows of two objects",
              " if they are not derived from the same thing!")
-    
+
     expr <- rep("", length(names(e1)))
     col.data_type <- rep("", length(names(e1)))
     col.udt_name <- rep("", length(names(e1)))
@@ -659,7 +862,7 @@ setMethod (
                              "]", sep = "")
             break
         }
-        
+
         for (k in seq_len(length(data.types)))
             if (e1@.col.data_type[i1] %in% data.types[[k]]) {
                 v <- k
@@ -672,11 +875,11 @@ setMethod (
             expr[i] <- "NULL"
         }
         col.data_type[i] <- res.type
-        col.udt_name[i] <- res.type
+        col.udt_name[i] <- res.udt
         col.name[i] <- paste(names(e1)[i1], "_", names(e2)[i2],
                              "_opr", sep = "")
     }
-    
+
     expr.str <- paste(expr, paste("\"", col.name, "\"", sep = ""),
                       sep = " as ", collapse = ", ")
     new("db.Rquery",
@@ -693,7 +896,8 @@ setMethod (
         .where = e1@.where,
         .is.factor = rep(FALSE, l),
         .factor.suffix = rep("", l),
-        .sort = sort)
+        .sort = sort,
+        .dist.by = e1@.dist.by)
 }
 
 ## -----------------------------------------------------------------------
@@ -702,8 +906,17 @@ setMethod (
     "+",
     signature(e1 = "db.obj", e2 = "db.obj"),
     function (e1, e2) {
+        if (is(e2, "db.Rquery") && e2@.is.agg) {
+            e2 <- unlist(lk(e2))
+            return (e1 + e2)
+        }
+        if (is(e1, "db.Rquery") && e1@.is.agg) {
+            e1 <- unlist(lk(e1))
+            return (e1 + e2)
+        }
         .operate.two(e1, e2, " + ", list(.num.types),
-                     res.type = "double precision")
+                     res.type = "double precision",
+                     res.udt = "float8")
     },
     valueClass = "db.Rquery")
 
@@ -713,8 +926,17 @@ setMethod (
     "-",
     signature(e1 = "db.obj", e2 = "db.obj"),
     function (e1, e2) {
+        if (is(e2, "db.Rquery") && e2@.is.agg) {
+            e2 <- unlist(lk(e2))
+            return (e1 - e2)
+        }
+        if (is(e1, "db.Rquery") && e1@.is.agg) {
+            e1 <- unlist(lk(e1))
+            return (e1 - e2)
+        }
         .operate.two(e1, e2, " - ", list(.num.types),
-                     res.type = "double precision")
+                     res.type = "double precision",
+                     res.udt = "float8")
     },
     valueClass = "db.Rquery")
 
@@ -724,8 +946,43 @@ setMethod (
     "*",
     signature(e1 = "db.obj", e2 = "db.obj"),
     function (e1, e2) {
-        .operate.two(e1, e2, " * ", list(.num.types),
-                     res.type = "double precision")
+        if (is(e2, "db.Rquery") && e2@.is.agg) {
+            e2 <- unlist(lk(e2))
+            return (e1 * e2)
+        }
+        if (is(e1, "db.Rquery") && e1@.is.agg) {
+            e1 <- unlist(lk(e1))
+            return (e1 * e2)
+        }
+        if (length(names(e1)) == 1 && e1@.col.data_type == "array" &&
+            length(names(e2)) == 1 && e2@.col.data_type %in% .num.types) {
+            if (! conn.eql(conn.id(e1), conn.id(e2)))
+                stop("The two objects are not in the same database!")
+            if (!.eql.parent(e1, e2))
+                stop("x and y cannot match because they originate",
+                     " from different sources!")
+            e1 <- e1[[names(e1)]]
+            e2 <- e2[,]
+            res <- e1
+            madlib <- schema.madlib(conn.id(e1))
+            res@.expr <- paste(madlib, ".array_scalar_mult(", e1@.expr,
+                               "::double precision[], (",
+                               e2@.expr, ")::double precision)", sep = "")
+            res@.col.name <- paste(e1@.col.name, "_opr", sep = "")
+            ## res@.content <- gsub("^select [^((?! as ).)]+\\S+ as",
+            res@.content <- gsub("^select .* as",
+                                 paste("select ", res@.expr, " as", sep = ""),
+                                 e1@.content)
+            return (res)
+        } else if (length(names(e2)) == 1 && e2@.col.data_type == "array" &&
+                   length(names(e1)) == 1 &&
+                   e1@.col.data_type %in% .num.types) {
+            return (e2 * e1)
+        } else {
+            .operate.two(e1, e2, " * ", list(.num.types),
+                         res.type = "double precision",
+                         res.udt = "float8")
+        }
     },
     valueClass = "db.Rquery")
 
@@ -735,8 +992,17 @@ setMethod (
     "/",
     signature(e1 = "db.obj", e2 = "db.obj"),
     function (e1, e2) {
+        if (is(e2, "db.Rquery") && e2@.is.agg) {
+            e2 <- unlist(lk(e2))
+            return (e1 / e2)
+        }
+        if (is(e1, "db.Rquery") && e1@.is.agg) {
+            e1 <- unlist(lk(e1))
+            return (e1 / e2)
+        }
         .operate.two(e1, e2, " / ", list(.num.types),
-                     res.type = "double precision")
+                     res.type = "double precision",
+                     res.udt = "float8")
     },
     valueClass = "db.Rquery")
 
@@ -746,8 +1012,17 @@ setMethod (
     "^",
     signature(e1 = "db.obj", e2 = "db.obj"),
     function (e1, e2) {
+        if (is(e2, "db.Rquery") && e2@.is.agg) {
+            e2 <- unlist(lk(e2))
+            return (e1^e2)
+        }
+        if (is(e1, "db.Rquery") && e1@.is.agg) {
+            e1 <- unlist(lk(e1))
+            return (e1^e2)
+        }
         .operate.two(e1, e2, " ^ ", list(.num.types),
-                     res.type = "double precision")
+                     res.type = "double precision",
+                     res.udt = "float8")
     },
     valueClass = "db.Rquery")
 
@@ -757,8 +1032,18 @@ setMethod (
     "%%",
     signature(e1 = "db.obj", e2 = "db.obj"),
     function (e1, e2) {
+        if (is(e2, "db.Rquery") && e2@.is.agg) {
+            e2 <- unlist(lk(e2))
+            return (e1 %% e2)
+        }
+        if (is(e1, "db.Rquery") && e1@.is.agg) {
+            e1 <- unlist(lk(e1))
+            return (e1 %% e2)
+        }
         .operate.two(e1, e2, " % ", list(.num.types),
-                     res.type = "double precision", cast = "::integer")
+                     res.type = "double precision",
+                     cast = "::integer",
+                     res.udt = "float8")
     },
     valueClass = "db.Rquery")
 
@@ -768,8 +1053,18 @@ setMethod (
     "%/%",
     signature(e1 = "db.obj", e2 = "db.obj"),
     function (e1, e2) {
+        if (is(e2, "db.Rquery") && e2@.is.agg) {
+            e2 <- unlist(lk(e2))
+            return (e1 %/% e2)
+        }
+        if (is(e1, "db.Rquery") && e1@.is.agg) {
+            e1 <- unlist(lk(e1))
+            return (e1 %/% e2)
+        }
         .operate.two(e1, e2, " / ", list(.int.types),
-                     res.type = "integer", cast = "::integer")
+                     res.type = "integer",
+                     cast = "::integer",
+                     res.udt = "int4")
     },
     valueClass = "db.Rquery")
 
@@ -779,6 +1074,14 @@ setMethod (
     ">",
     signature(e1 = "db.obj", e2 = "db.obj"),
     function (e1, e2) {
+        if (is(e2, "db.Rquery") && e2@.is.agg) {
+            e2 <- unlist(lk(e2))
+            return (e1 > e2)
+        }
+        if (is(e1, "db.Rquery") && e1@.is.agg) {
+            e1 <- unlist(lk(e1))
+            return (e1 > e2)
+        }
         .operate.two(e1, e2, " > ", list(.num.types, .txt.types),
                      res.type = "boolean", cast = "")
     },
@@ -790,6 +1093,14 @@ setMethod (
     "<",
     signature(e1 = "db.obj", e2 = "db.obj"),
     function (e1, e2) {
+        if (is(e2, "db.Rquery") && e2@.is.agg) {
+            e2 <- unlist(lk(e2))
+            return (e1 < e2)
+        }
+        if (is(e1, "db.Rquery") && e1@.is.agg) {
+            e1 <- unlist(lk(e1))
+            return (e1 < e2)
+        }
         .operate.two(e1, e2, " < ", list(.num.types, .txt.types),
                      res.type = "boolean", cast = "")
     },
@@ -801,6 +1112,14 @@ setMethod (
     ">=",
     signature(e1 = "db.obj", e2 = "db.obj"),
     function (e1, e2) {
+        if (is(e2, "db.Rquery") && e2@.is.agg) {
+            e2 <- unlist(lk(e2))
+            return (e1 >= e2)
+        }
+        if (is(e1, "db.Rquery") && e1@.is.agg) {
+            e1 <- unlist(lk(e1))
+            return (e1 >= e2)
+        }
         .operate.two(e1, e2, " >= ", list(.num.types, .txt.types),
                      res.type = "boolean", cast = "")
     },
@@ -812,6 +1131,14 @@ setMethod (
     "<=",
     signature(e1 = "db.obj", e2 = "db.obj"),
     function (e1, e2) {
+        if (is(e2, "db.Rquery") && e2@.is.agg) {
+            e2 <- unlist(lk(e2))
+            return (e1 <= e2)
+        }
+        if (is(e1, "db.Rquery") && e1@.is.agg) {
+            e1 <- unlist(lk(e1))
+            return (e1 <= e2)
+        }
         .operate.two(e1, e2, " <= ", list(.num.types, .txt.types),
                      res.type = "boolean", cast = "")
     },
@@ -823,6 +1150,14 @@ setMethod (
     "==",
     signature(e1 = "db.obj", e2 = "db.obj"),
     function (e1, e2) {
+        if (is(e2, "db.Rquery") && e2@.is.agg) {
+            e2 <- unlist(lk(e2))
+            return (e1 == e2)
+        }
+        if (is(e1, "db.Rquery") && e1@.is.agg) {
+            e1 <- unlist(lk(e1))
+            return (e1 == e2)
+        }
         .operate.two(e1, e2, " = ", list(.num.types, .txt.types, "boolean"),
                      res.type = "boolean", cast = "")
     },
@@ -834,6 +1169,14 @@ setMethod (
     "!=",
     signature(e1 = "db.obj", e2 = "db.obj"),
     function (e1, e2) {
+        if (is(e2, "db.Rquery") && e2@.is.agg) {
+            e2 <- unlist(lk(e2))
+            return (e1 != e2)
+        }
+        if (is(e1, "db.Rquery") && e1@.is.agg) {
+            e1 <- unlist(lk(e1))
+            return (e1 != e2)
+        }
         .operate.two(e1, e2, " <> ", list(.num.types, .txt.types, "boolean"),
                      res.type = "boolean", cast = "")
     },
@@ -845,6 +1188,14 @@ setMethod (
     "&",
     signature(e1 = "db.obj", e2 = "db.obj"),
     function (e1, e2) {
+        if (is(e2, "db.Rquery") && e2@.is.agg) {
+            e2 <- unlist(lk(e2))
+            return (e1 & e2)
+        }
+        if (is(e1, "db.Rquery") && e1@.is.agg) {
+            e1 <- unlist(lk(e1))
+            return (e1 & e2)
+        }
         .operate.two(e1, e2, " and ", list(c("boolean")),
                      res.type = "boolean", cast = "")
     },
@@ -856,6 +1207,14 @@ setMethod (
     "|",
     signature(e1 = "db.obj", e2 = "db.obj"),
     function (e1, e2) {
+        if (is(e2, "db.Rquery") && e2@.is.agg) {
+            e2 <- unlist(lk(e2))
+            return (e1 | e2)
+        }
+        if (is(e1, "db.Rquery") && e1@.is.agg) {
+            e1 <- unlist(lk(e1))
+            return (e1 | e2)
+        }
         .operate.two(e1, e2, " or ", list(c("boolean")),
                      res.type = "boolean", cast = "")
     },
@@ -880,6 +1239,45 @@ setMethod (
     "is.na",
     signature(x = "db.obj"),
     function (x) {
-        .compare(x, "", " is NULL", NA, "", "boolean", "")
+        ## if (array)
+        ##     .compare(x, "", " is NULL", NA, "", "boolean", "",
+        ##              restore.array = FALSE)
+        ## else
+        res <- .compare(x, "", " is NULL", NA, "", "boolean", "")
+        if (length(x@.col.data_type) == 1 && x@.col.data_type == "array") {
+            .check.madlib.version(x, 1.3)
+            madlib <- schema.madlib(conn.id(x))
+            tmp <- paste(res@.expr, " or ", madlib,
+                         ".array_contains_null(", x@.expr, ")", sep="")
+            ## res@.content <- gsub("^select [^((?! as ).)]+\\S+ as",
+            res@.content <- gsub("^select .* as",
+                                 paste("select", tmp, "as"),
+                                 res@.content)
+            res@.expr <- tmp
+        }
+        res
     },
     valueClass = "db.Rquery")
+
+## ----------------------------------------------------------------------
+
+setGeneric("grepl")
+
+setMethod(
+    "grepl",
+    signature(pattern = "character", x = "db.obj"),
+    function (pattern, x, ignore.case = FALSE, perl = FALSE, fixed = FALSE,
+              useBytes = FALSE)
+{
+    pattern <- paste("'", .strip(as.character(pattern), "'"), "'", sep = "")
+    if (fixed) pattern <- gsub("\\", "\\\\", pattern)
+    if (ignore.case) cmp <- "~*"
+    else cmp <- "~"
+    res <- .compare(x, pattern, cmp, .txt.types, cast = "")
+    res
+})
+
+## ----------------------------------------------------------------------
+
+`%+%` <- function (e1, e2) paste(e1, e2, sep = "")
+`%.%` <- function (e1, e2) paste(e2, collapse = e1)
