@@ -73,9 +73,10 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
 ## get the distributed by string
 .get.distributed.by.str <- function(conn.id, distributed.by)
 {
-    dbms.str <- dbms(conn.id)
-    if (gsub(".*(Greenplum).*", "\\1", dbms.str, perl=T) == "Greenplum") {
-        if (is.null(distributed.by))
+    dbms.str <- (.get.dbms.str(conn.id))$db.str
+    if (dbms.str != "PostgreSQL") {
+        if (is.null(distributed.by) ||
+            identical(distributed.by, character(0)))
             dist.str <- "DISTRIBUTED RANDOMLY"
         else {
             if (!.is.arg.string(distributed.by))
@@ -169,7 +170,8 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
 
 .unique.string <- function ()
 {
-    r1 <- sample(seq_len(100000000), 1)
+    ## r1 <- sample(seq_len(100000000), 1)
+    r1 <- as.integer(runif(1) * 1e8) + 1
     r2 <- unclass(as.POSIXct(strptime(date(),"%c")))[1]
     r3 <- r2 %% r1
     paste("__madlib_temp_", r1, "_", r2, "_", r3, "__", sep = "")
@@ -188,44 +190,62 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
 
 ## analyze formula
 .analyze.formula <- function (formula, data, fdata = data, refresh = FALSE,
-                              is.factor = NA, cols = NA, suffix = NA)
+                              is.factor = NA, cols = NA, suffix = NA,
+                              grp.vars = NULL, grp.expr = NULL)
 {    
     f.str <- strsplit(paste(deparse(formula), collapse = ""), "\\|")[[1]]
-    f.str <- .replace.array(f.str, data)
+    ## f.str <- .replace.array(f.str, data)
     fstr <- f.str[1]
-    f1 <- formula(fstr) # formula
+    
     f2 <- f.str[2] # grouping columns, might be NA
 
-    fdata <- .expand.array(fdata)
-    
-    if (!is.na(f2)) {
+    ## fdata <- .expand.array(fdata)
+    if (!is.null(grp.vars) && !is.na(f2)) {
+        f2.labels <- grp.vars
+        grp.expr <- grp.expr
+        grp <- paste(f2.labels, collapse = ", ")
+    } else if (!is.na(f2)) {
         f2.terms <- terms(formula(paste("~", f2)))
         f2.labels <- attr(f2.terms, "term.labels")
-        inter <- intersect(f2.labels, names(fdata))
-        if (length(inter) != length(f2.labels))
-            stop("The grouping part of the formula is not quite right!")
-        ## grouping column do not use factor
+        ## inter <- intersect(f2.labels, names(fdata))
+        ## if (length(inter) != length(f2.labels))
+        ##     stop("The grouping part of the formula is not quite right!")
+        ## ## grouping column do not use factor
+        f2.labels <- gsub("I\\((.*)\\)", "\\1", f2.labels, perl = T)
+        f2.labels <- gsub("as.factor\\((.*)\\)", "\\1", f2.labels, perl = T)
         f2.labels <- gsub("factor\\((.*)\\)", "\\1", f2.labels, perl = T)
-        f2.labels <- gsub("factor\\((.*)\\)", "\\1", f2.labels, perl = T)
-        f2.labels <- .replace.with.quotes(f2.labels, data@.col.name)
-        ## f2.labels <- gsub("`([^`]*)(\\[\\d+\\])`", "\"\\1\"\\2", f2.labels)
+        ## f2.labels <- .replace.with.quotes(f2.labels, data@.col.name)
+        ## ## f2.labels <- gsub("`([^`]*)(\\[\\d+\\])`", "\"\\1\"\\2", f2.labels)
+        grp.expr <- f2.labels
+        for (i in seq_len(length(f2.labels))) {
+            if (! f2.labels[i] %in% names(data)) {
+                grp.col <- .unique.string()
+                data[[grp.col]] <- eval(parse(text = paste("with(data, ",
+                                              f2.labels[i], ")", sep = "")))
+                f2.labels[i] <- grp.col
+            }                
+        }
         grp <- paste(f2.labels, collapse = ", ")
     } else {
         f2.labels <- NULL
         grp <- NULL
     }
 
+    if (!is.null(f2.labels))
+        fstr <- paste(fstr, "-", paste("`", f2.labels, "`",
+                                       collapse = "-", sep = ""))
+
     ## create a fake data.frame only to extract
     ## terms when there is "." in formula
     fake.data <- structure(vector("list", ncol(fdata)), names=names(fdata), class="data.frame")
+    f1 <- formula(fstr) # formula
     f.terms <- terms(f1, data = fake.data) # formula terms
     ## the 1st row is the dependent variable
     f.factors <- attr(f.terms, "factors")
     f.labels <- attr(f.terms, "term.labels") # each terms on the right side
     ## f.labels <- gsub("`([^`]*)(\\[\\d+\\])`", "\"\\1\"\\2", f.labels)    
-    
     right.hand <- paste(f.labels, collapse = "+")
-    if (refresh) {
+    if (refresh) { # second pass
         replace.cols <- cols[is.factor]
         suffix <- suffix[is.factor]
         n.order <- order(nchar(replace.cols), decreasing = TRUE)
@@ -235,14 +255,17 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
             col <- replace.cols[i]
             new.col <- names(data)[grep(paste(col, suffix[i], sep=""),
                                         names(data))]
-            new.col <- paste("(", paste("`", new.col, "`", collapse = " + ",
-                                        sep = ""), ")",
-                             sep = "")
+            if (identical(new.col, character(0))) 
+                new.col <- "1"
+            else
+                new.col <- paste("(", paste("`", new.col, "`",
+                                            collapse = " + ",
+                                            sep = ""), ")", sep = "")
             right.hand <- gsub(paste(col, "([^_\\w]+|$)", sep = ""),
                                paste(new.col, "\\1", sep = ""),
                                right.hand, perl = TRUE)
         }
-    } else {
+    } else { # first pass
         ## # make all text columns to be factor
         ## for (i in seq_len(length(names(data)))) {
         ##     if (data@.col.data_type[i] %in% .txt.types) {
@@ -276,21 +299,30 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
 
     right.hand <- gsub("as\\.factor\\((((?!as\\.factor).)*)\\)", "\\1", right.hand, perl = T)
     right.hand <- gsub("factor\\((((?!factor).)*)\\)", "\\1", right.hand, perl = T)
-
     f.terms1 <- terms(formula(paste("~", right.hand)), data = fake.data)
     f.labels <- attr(f.terms1, "term.labels")
     ## f.labels <- gsub("`([^`]*)(\\[\\d+\\])`", "\"\\1\"\\2", f.labels)
 
     f.intercept <- attr(f.terms, "intercept")
-    labels <- gsub(":", "*", f.labels, perl = T) # replace interaction : with *
+    labels <- gsub("\\[(\\d+):(\\d+)\\]", "[\\1@\\2]", f.labels)
+    labels <- gsub(":", "*", labels, perl = T) # replace interaction : with *
+    labels <- gsub("\\[(\\d+)@(\\d+)\\]", "[\\1:\\2]", labels)
     labels <- gsub("I\\((.*)\\)", "\\1", labels, perl = T) # remove I()
 
+    ## vdata <- .expand.array(data)
+    vdata <- data
+
     ## dependent variable
     ## factor does not play a role in dependent variable
     dep.var <- gsub("I\\((.*)\\)", "\\1", rownames(f.factors)[1], perl = T)
     dep.var <- gsub("as.factor\\((.*)\\)", "\\1", dep.var, perl = T)
     dep.var <- gsub("factor\\((.*)\\)", "\\1", dep.var, perl = T)
-    dep.var <- .replace.with.quotes(dep.var, data@.col.name)
+    ## dep.var <- .replace.with.quotes(dep.var, data@.col.name)
+    origin.dep <- dep.var
+
+    tmp <- eval(parse(text = paste("with(vdata, ", dep.var, ")", sep = "")))
+    dep.var <- tmp@.expr
+
     ## dep.var <- gsub("`([^`]*)(\\[\\d+\\])`", "\"\\1\"\\2", dep.var)
 
     ## with or without intercept
@@ -300,11 +332,26 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
         intercept.str <- "1,"
 
     labels <- .is.array(labels, data)
+    orig.labels <- labels
 
-    labels <- .replace.with.quotes(labels, data@.col.name)
+    a <- eval(parse(text = paste("with(vdata, c(",
+                    paste(labels, collapse = ", "), "))",
+                    sep = "")))
+    a.labels <- unlist(sapply(a, function(x) x[,]@.expr))
+
+    b <- eval(parse(text = paste("with(vdata, c(",
+                    paste(setdiff(rownames(f.factors),
+                                  colnames(f.factors)),
+                          collapse = ", "), "))", sep = "")))
+    b.labels <- unlist(sapply(b, function(x) x[,]@.expr))
+
+    labels <- .strip(setdiff(a.labels, b.labels), "`")
+    
+    ## labels <- .replace.with.quotes(labels, data@.col.name)
     ## remove grouping columns, when there is no intercept term
     if (!is.null(f2.labels) && f.intercept != 0)
-        labels <- setdiff(labels, f2.labels)
+        labels <- setdiff(labels, .strip(f2.labels, "`"))
+    
     ind.var <- paste("array[", intercept.str,
                      paste(labels, collapse = ","),
                      "]", sep = "") # independent variable
@@ -324,7 +371,10 @@ arraydb.to.arrayr <- function (str, type = "double", n = 1)
     labels <- gsub("`", "", labels)
     ##
     
-    list(dep.str = dep.var, ind.str = ind.var, grp.str = grp,
+    list(dep.str = dep.var, origin.dep = origin.dep,
+         origin.ind = orig.labels,
+         ind.str = ind.var,
+         grp.str = grp, grp.vars = f2.labels, grp.expr = grp.expr,
          ind.vars = labels,
          has.intercept = as.logical(f.intercept),
          data = data,
